/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : Assert
   date       : 23.02.98
   description: a class that acts like the C/C++ assertion macro
                For a detailed description of this topic look at
                "http://www.afu.com/javafaq.html" and search for
                "11.6 How can I write C/C++ style assertions in Java?"
   modified   : 03.03.98 - static variable "enabled" and
                           AssertionException added

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public final class Assert {

  // class variable declarations
  public static final boolean enabled = /* true */ false ;
  public static PrintWriter out;

  // static initializers
  static {
    if (enabled) {
      out = new PrintWriter(System.out,true);
    }
    else {
      out = new PrintWriter(new AssertionOutputStream(System.out),false);
    } // end if
  } // end static initializer

  // methods
  public static final void assert(boolean b) {
    if (enabled && !b) throw new AssertionException();
  } // end assert

  public static final void assert(boolean b, String s) {
    if (enabled && !b) throw new AssertionException(s);
  } // end assert

  public static final void assert(long l) {
    if (enabled && l == 0L) throw new AssertionException();
  } // end assert

  public static final void assert(long l, String s) {
    if (enabled && l == 0L) throw new AssertionException(s);
  } // end assert

  public static final void assert(Object ref) {
    if (enabled && ref == null) throw new AssertionException();
  } // end assert

  public static final void assert(Object ref, String s) {
    if (enabled && ref == null) throw new AssertionException(s);
  } // end assert

} // end Assert

/* ******************************************************************
   top level class declaration
*********************************************************************/

class AssertionException extends Error {

  // constructors
  public AssertionException() {
    super();
  } // end AssertionException

  public AssertionException(String s) {
    super(s);
  } // end AssertionException

} // end AssertionException

// ******************************************************************

class AssertionOutputStream extends FilterOutputStream {

  // constructors
  public AssertionOutputStream(OutputStream out) {
    super(out);
  } // end constructor AssertionOutputStream

  // methods
  public void write(byte b[]) {}

  public void write(byte b[], int off, int len) {}

  public void write(int b) {}

} // end AssertionOutputStream

/* ******************************************************************
   end of file
*********************************************************************/
