/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : FindDateDlg
   date       : 20.02.98
   description: a dialog that allows to specify settings for search
                of free dates
   modified   : 13.03.98 - dialog is no longer initially visible

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*; 
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class FindDateDlg extends Dialog implements ActionListener {

  // class variable declarations
  private final static int SEARCHRANGE_UNIT   = Calendar.MONTH;
  private final static int SEARCHRANGE_WIDTH  = 2;
  private final static String DEF_REQUIRETIME = "1:00";
  private final static String DEF_STARTTIME   = "8:00";
  private final static String DEF_ENDTIME     = "17:00";

  // instance variable declarations
  private Frame parent;
  private boolean canceled    = false;
  private boolean autoCancel  = false;
  private boolean autoDisable = false;
  private Button okButton;
  private Button cancelButton;
  private TextField startDateText;
  private TextField endDateText;
  private Button startDateSelBtn;
  private Button endDateSelBtn;
  private TextField requireTimeText;
  private TextField startTimeText;
  private TextField endTimeText;
  private List personsList;
  private Vector personsVector;
  private String creator;
  private Calendar calendar;
  private Vector resultVector;
  private int min,hour;
  private SimpleDateFormat dateFormatter;

  // inner classes
  class FindDateDlgListener extends WindowAdapter {

    // methods
    public void windowClosing(WindowEvent e) {
      setCanceled(true);
      setVisible(false);
      dispose();
    } // end windowClosing

    public void windowActivated(WindowEvent e) {
    } // end WindowActivated

    public void windowDeactivated(WindowEvent e) {
      if (autoCancel) {
        setCanceled(true);
        setVisible(false);
        dispose();
      } // end if
    } // end WindowDeactivated

  } // end FindDateDlgListener

  class DateListener extends WindowAdapter {

    private DateChooserDlg dateDlg;
    private boolean secondBtn;

    public DateListener(DateChooserDlg dateDlg, boolean secondBtn) {
      super();
      this.dateDlg   = dateDlg;
      this.secondBtn = secondBtn;
    } // end constructor DateListener

    public void windowClosed(WindowEvent e) {
      if (autoDisable) { autoCancel = true; autoDisable = false; }
      if (!dateDlg.getCanceled()) {
        if (secondBtn)
          endDateText.setText(dateFormatter.format(dateDlg.getDate()));
        else
          startDateText.setText(dateFormatter.format(dateDlg.getDate()));
        // end if
      } // end if
    } // end windowClosed
  
  } // end DateListener

  class MsgListener extends WindowAdapter {

    public void windowClosed(WindowEvent e) {
      autoCancel  = true;
      autoDisable = false;
    } // end windowClosed
  
  } // end DateListener

  // constructors
  public FindDateDlg(Frame parent, String[] persons, String creator) {
    super(parent);
    this.parent = parent;
    init(persons,creator);
  } // end constructor FindDateDlg

  public FindDateDlg(Frame parent, boolean modal, String[] persons,
                     String creator) {
    super(parent,modal);
    this.parent = parent;
    init(persons,creator);
  } // end constructor FindDateDlg

  public FindDateDlg(Frame parent, String title, String[] persons,
                     String creator) {
    super(parent,title);
    this.parent = parent;
    init(persons,creator);
  } // end constructor FindDateDlg

  public FindDateDlg(Frame parent, String title, boolean modal,
                     String[] persons, String creator) {
    super(parent,title,modal);
    this.parent = parent;
    init(persons,creator);
  } // end constructor FindDateDlg

  // methods
  private void init(String[] persons, String creator) {
    int i,j,n;

    this.personsVector = new Vector();
    for (i=0; i<persons.length; i++) personsVector.addElement(persons[i]);
    this.creator = creator;
    resultVector = new Vector();

    addWindowListener(new FindDateDlgListener());
    dateFormatter = new SimpleDateFormat("dd.MM.yy");

    // this dialog box uses the gridbaglayout to provide a pretty good layout
    GridBagLayout gridbag          = new GridBagLayout(); 
    GridBagConstraints constraints = new GridBagConstraints();

    // create the widgets
    InsetsPanel pan = new InsetsPanel(new Insets(15,20,15,20));
    okButton = new Button("OK");
    okButton.addActionListener(this);
    cancelButton = new Button("Cancel");
    cancelButton.addActionListener(this);
    InsetsPanel buttonPanel = new InsetsPanel(new Insets(15,0,0,0));
    buttonPanel.setLayout(new GridLayout(1,2,10,0));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);

    // set the calendar to the date of tomorrow
    calendar = CalendarCreator.get();
    calendar.setTime(new Date());
    calendar.add(Calendar.DATE,1);
    Label startDateLabel = new Label("Start Date:");
    startDateText = new TextField(dateFormatter.format(calendar.getTime()),8);
    startDateSelBtn = new Button("Select");
    startDateSelBtn.addActionListener(this);

    // set the calendar to the end of the search range
    calendar.add(SEARCHRANGE_UNIT,SEARCHRANGE_WIDTH);
    Label endDateLabel = new Label("End Date:");
    endDateText = new TextField(dateFormatter.format(calendar.getTime()),8);
    endDateSelBtn = new Button("Select");
    endDateSelBtn.addActionListener(this);

    Label requireTimeLabel = new Label("Time Require:");
    requireTimeText = new TextField(DEF_REQUIRETIME,5);

    Label startTimeLabel = new Label("Start Time:");
    startTimeText = new TextField(DEF_STARTTIME,5);

    Label endTimeLabel = new Label("End Time:");
    endTimeText = new TextField(DEF_ENDTIME,5);

    // group the time labels together
    Panel timeLabelPanel = new Panel(new GridLayout(3,1));
    timeLabelPanel.add(requireTimeLabel);
    timeLabelPanel.add(startTimeLabel);
    timeLabelPanel.add(endTimeLabel);

    // group the time texts together
    Panel timeTextPanel = new Panel(new GridLayout(3,1));
    timeTextPanel.add(requireTimeText);
    timeTextPanel.add(startTimeText);
    timeTextPanel.add(endTimeText);

    // set up the personsList
    String person;

    personsList = new List(6,true);
    n           = personsVector.size();
    for (i=0; i<n; i++) {
      person = (String)personsVector.elementAt(i);
      if (!creator.equals(person)) personsList.add(person);
    } // end for

    pan.setLayout(gridbag); 

    constraints.insets     = new Insets(5,6,5,6);
    constraints.fill       = GridBagConstraints.NONE; 
    constraints.anchor     = GridBagConstraints.WEST; 
    constraints.weightx    = 1.0; 
    constraints.weighty    = 1.0; 
    constraints.gridwidth  = 1;
    constraints.gridheight = 1; 
    gridbag.setConstraints(startDateLabel,constraints); 
    pan.add(startDateLabel);

    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(startDateText,constraints); 
    pan.add(startDateText);

    constraints.anchor     = GridBagConstraints.WEST; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(startDateSelBtn,constraints); 
    pan.add(startDateSelBtn);

    constraints.gridwidth  = 1;
    gridbag.setConstraints(endDateLabel,constraints); 
    pan.add(endDateLabel);

    constraints.anchor     = GridBagConstraints.CENTER; 
    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(endDateText,constraints); 
    pan.add(endDateText);

    constraints.anchor     = GridBagConstraints.WEST; 
    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(endDateSelBtn,constraints); 
    pan.add(endDateSelBtn);

    SeparatorCanvas separator = new SeparatorCanvas();
    constraints.fill       = GridBagConstraints.HORIZONTAL; 
    constraints.anchor     = GridBagConstraints.CENTER; 
    gridbag.setConstraints(separator,constraints); 
    pan.add(separator);

    constraints.fill       = GridBagConstraints.NONE;
    constraints.gridwidth  = 1;
    gridbag.setConstraints(timeLabelPanel,constraints); 
    pan.add(timeLabelPanel);

    constraints.gridwidth  = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(timeTextPanel,constraints); 
    pan.add(timeTextPanel);

    constraints.gridwidth  = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(personsList,constraints); 
    pan.add(personsList); 

    separator = new SeparatorCanvas();
    constraints.fill       = GridBagConstraints.HORIZONTAL; 
    constraints.gridheight = GridBagConstraints.RELATIVE;
    gridbag.setConstraints(separator,constraints); 
    pan.add(separator);

    constraints.fill       = GridBagConstraints.NONE;
    constraints.gridheight = GridBagConstraints.REMAINDER; 
    gridbag.setConstraints(buttonPanel, constraints); 
    pan.add(buttonPanel); 

    add(pan);
    pack(); 
    setResizable(false);
    //setVisible(true);
  } // end init

  public Vector getResultVector() {
    return resultVector;
  } // end getResultVector

  public int getMinutesReqire() {
    return min;
  } // end getMinutesReqire

  public int getHoursReqire() {
    return hour;
  } // end getHoursReqire

  public CalendarEntry getCalendarEntry() {
    CalendarEntry entry = new CalendarEntry(creator);
    entry.setPersons(personsList.getSelectedItems());
    return entry;
  } // end getCalendarEntry

  public String[] getPersons() {
    String[] tmp     = personsList.getSelectedItems();
    String[] persons = new String[tmp.length+1];
    int i;

    for (i=0; i<tmp.length; i++) persons[i] = tmp[i];
    persons[tmp.length] = creator;
    return persons;
  } // end getPersons
    
  private void makeMsgBox(String str) {
    MsgBoxDlg msgBoxDlg = new MsgBoxDlg(parent,getTitle(),isModal(),str,
                                MsgBoxDlg.OK | MsgBoxDlg.ICONINFORMATION);
    if (autoCancel) {
      autoCancel = false;
      autoDisable = true;
      msgBoxDlg.addWindowListener(new MsgListener());
      msgBoxDlg.setAutoCancel(true);
    } // end if
  } // end makeMsgBox

  private boolean readResults() {
    Date date,start,end,daylyStart,daylyEnd;
    ParsePosition pos                  = new ParsePosition(0);
    SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("dd.MM.yy.HH:mm:ss");
    String str = startDateText.getText()+".00:00:00";

    // read the start date
    try {
      date = dateTimeFormatter.parse(str,pos);
    } catch (NullPointerException e) { date = null; }
    if (date == null) {
      startDateText.setText(""); startDateText.requestFocus();
      makeMsgBox("Please insert the start date as 'dd.mm.yy'.");
      return false;
    } // end if
    start = new Date(date.getTime());
    // read the start date
    pos.setIndex(0);
    str = endDateText.getText()+".00:00:00";
    try {
      date = dateTimeFormatter.parse(str,pos);
    } catch (NullPointerException e) { date = null; }
    if (date == null) {
      endDateText.setText(""); endDateText.requestFocus();
      makeMsgBox("Please insert the end date as 'dd.mm.yy'.");
      return false;
    } // end if
    end = new Date(date.getTime());
    // check if end is after start
    if (end.before(start)) {
      endDateText.setText(""); endDateText.requestFocus();
      makeMsgBox("Please insert a end date that is before the start date.");
      return false;
    } // end if
    // add one day to the end date
    calendar.setTime(end);
    calendar.add(Calendar.DATE,1);
    end.setTime(calendar.getTime().getTime()); 
    // read the time require
    pos.setIndex(0);
    str = startDateText.getText()+".00:"+requireTimeText.getText();
    try {
      calendar.setTime(dateTimeFormatter.parse(str,pos));
      // hours are moved to minutes, minutes are moved to seconds, so it
      // is possible to read a value of 24 hours (very tricky!)
      hour = calendar.get(Calendar.MINUTE);
      min  = calendar.get(Calendar.SECOND);
    } catch (NullPointerException e) { hour = 0; min = 0; }
    if ((60*hour+min < 1) || (60*hour+min > 24*60)) {
      requireTimeText.setText(""); requireTimeText.requestFocus();
      makeMsgBox("Please insert a time require between one minute\n"+
                 "and 24 hours as 'hh:mm'.");
      return false;
    } // end if
    // read the start time
    pos.setIndex(0);
    str = startDateText.getText()+"."+startTimeText.getText()+":00";
    try {
      date = dateTimeFormatter.parse(str,pos);
    } catch (NullPointerException e) { date = null; }
    if (date == null) {
      startTimeText.setText(""); startTimeText.requestFocus();
      makeMsgBox("Please insert the start time as 'hh:mm'.");
      return false;
    } // end if
    daylyStart = new Date(date.getTime());
    // read the end time
    pos.setIndex(0);
    str = startDateText.getText()+"."+endTimeText.getText()+":00";
    try {
      date = dateTimeFormatter.parse(str,pos);
    } catch (NullPointerException e) { date = null; }
    if (date == null) {
      endTimeText.setText(""); endTimeText.requestFocus();
      makeMsgBox("Please insert the end time as 'hh:mm'.");
      return false;
    } // end if
    daylyEnd = new Date(date.getTime());
    // if the end time is before the start time,
    // then the start must be one day before
    if (daylyEnd.before(daylyStart)) {
      calendar.setTime(daylyStart);
      calendar.add(Calendar.DATE,-1);
      daylyStart.setTime(calendar.getTime().getTime()); 
    } // end if
    // persons settings
    //entry.setPersons(personsList.getSelectedItems());

    // build up the result vector
    resultVector.removeAllElements();
    // if the time require is greater than the dayly time scope, then the
    // empty vector is the result
    date.setTime(daylyStart.getTime());
    calendar.setTime(date);
    calendar.add(Calendar.HOUR_OF_DAY,hour);
    calendar.add(Calendar.MINUTE,min);
    date.setTime(calendar.getTime().getTime());
    if ((date.after(daylyEnd)) &&
        (!daylyStart.equals(daylyEnd))) return true;
    // set the result vector to the full search range
    resultVector.addElement(new DateDupel(start.getTime(),end.getTime()));
    // if dayly start and dayly end is equal, then this is allready the result
    if (daylyStart.equals(daylyEnd)) return true;
    // calculate the time, where no date can be
    date.setTime(daylyEnd.getTime());
    daylyEnd.setTime(daylyStart.getTime());
    calendar.setTime(date);
    calendar.add(Calendar.DATE,-1);
    daylyStart.setTime(calendar.getTime().getTime());
    // cut out all periods of time, where no date can be
    DateDupel cur,rest,tmp;

    while (!daylyStart.after(end)) {
      rest = (DateDupel)resultVector.lastElement();
      cur  = new DateDupel(daylyStart.getTime(),daylyEnd.getTime());
      if (rest.startsIn(cur)) {
        rest.start.setTime(cur.end.getTime());
      }
      else if (cur.isIn(rest)) {
        tmp = new DateDupel(rest.start.getTime(),cur.start.getTime());
        rest.start.setTime(cur.end.getTime());
        resultVector.insertElementAt(tmp,resultVector.size()-1);
      }
      else if (cur.startsIn(rest)) {
        rest.end.setTime(cur.start.getTime());
      } // end if
      // increase dayly start and dayly end
      calendar.setTime(daylyStart);
      calendar.add(Calendar.DATE,1);
      daylyStart.setTime(calendar.getTime().getTime());
      calendar.setTime(daylyEnd);
      calendar.add(Calendar.DATE,1);
      daylyEnd.setTime(calendar.getTime().getTime());
    } // end while
    return true;
  } // end readResults

  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();

    if (cmd.equals("Cancel")) {
      canceled = true;
      setVisible(false);
      dispose();
    }
    else if (cmd.equals("Select")) {
      Calendar calendar = CalendarCreator.get();
      boolean secondBtn = (endDateSelBtn == (Button)e.getSource());
      if (autoCancel) { autoCancel = false; autoDisable = true; }
      DateChooserDlg dateDlg = new DateChooserDlg(parent,"Select Date",
                                                  isModal(),calendar,
                                                  CalendarCreator.getLocale());
      dateDlg.addWindowListener(new DateListener(dateDlg,secondBtn));
      dateDlg.setVisible(true);
    }
    else { // if cmd equals Ok
      if (readResults()) {
        setVisible(false);
        dispose();
      } // end if
    } // end if
  } // end actionPerformed

  public boolean getCanceled() {
    return canceled;
  } // end getCanceled

  public void setCanceled(boolean canceled) {
    this.canceled = canceled;
  } // end getCanceled

  public boolean getAutoCancel() {
    return autoCancel;
  } // end getAutoCancel

  public void setAutoCancel(boolean autoCancel) {
    this.autoCancel = autoCancel;
  } // end getAutoCancel

} // end FindDateDlg

/* ******************************************************************
   end of file
*********************************************************************/
