/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : MonthlyPanel
   date       : 19.02.98
   description: an panel that displays the month view of the calendar

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class MonthlyPanel extends Panel
  implements Observer,ItemListener {

  // class variable declarations
  private final static int DATE_WIDTH         = 70;
  private final static int ENTRY_WIDTH        = 460;
  private final static int[] A_COL_WIDTH      = {DATE_WIDTH,ENTRY_WIDTH};

  // instance variable declarations
  private LoginInfoModel lModel;
  private ScopeModel sModel;
  private DateListModel dModel;
  private int vSize;
  private SpreadSheetCanvas spreadSheet;
  private SimpleDateFormat formatter;
  private Calendar calendar;
  private int nrOfDays;

  // constructors
  public MonthlyPanel(LoginInfoModel lModel, ScopeModel sModel,
                    DateListModel dModel, int vSize) {
    formatter = new SimpleDateFormat("EE, dd.MM.yy",CalendarCreator.getLocale());
    int i,curMonth;

    this.lModel = lModel;
    this.sModel = sModel;
    this.dModel = dModel;
    this.sModel.addObserver(this);
    this.dModel.addObserver(this);
    this.vSize = vSize;

    calendar = CalendarCreator.get();

    ScrollPane sp = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
    sp.setSize(DATE_WIDTH+ENTRY_WIDTH+sp.getVScrollbarWidth(),vSize);
    spreadSheet = new SpreadSheetCanvas(31,A_COL_WIDTH,
                                        SystemColor.control,true);
    sp.add(spreadSheet);
    calendar.setTime(sModel.getStartOfScope());
    curMonth = calendar.get(Calendar.MONTH);
    nrOfDays = 0;
    do {
      spreadSheet.setCell(0,nrOfDays,0,formatter.format(calendar.getTime()));
      calendar.add(Calendar.DATE,1);
      nrOfDays++;
    } while (calendar.get(Calendar.MONTH) == curMonth);
    // create empty column headers for month with less than 31 days
    for (i=nrOfDays; i<31; i++) {
      spreadSheet.setCell(0,i,0,"");
    } // end for
    add(sp);
    spreadSheet.addItemListener(this);
  } // end constructor MonthlyPanel

  // methods
  public void update(Observable obs, Object arg) {
    if (sModel.getScopeWidth() != ScopeModel.MONTHLY) {
      spreadSheet.setBufferEnabled(false);
      return;
    }
    else {
      spreadSheet.setBufferEnabled(true);
    } // end if

    if (obs.equals(dModel)) {
      if (dModel.isNew()) {
        // update spreadsheet
        Enumeration enum = dModel.getCalendarEntries();
        CalendarEntry entry;
        int i,l,curMonth;
        Date oldDate;

        // update column header
        calendar.setTime(sModel.getStartOfScope());
        curMonth = calendar.get(Calendar.MONTH);
        nrOfDays = 0;
        do {
          spreadSheet.setCell(0,nrOfDays,0,formatter.format(calendar.getTime()));
          calendar.add(Calendar.DATE,1);
          nrOfDays++;
        } while (calendar.get(Calendar.MONTH) == curMonth);
        // create empty column headers for month with less than 31 days
        for (i=nrOfDays; i<31; i++) {
          spreadSheet.setCell(0,i,0,"");
        } // end for

        // delete spreadsheet
        spreadSheet.unSelect();
        for (i=0; i<31; i++) spreadSheet.deleteCell(1,i);

        // set calendar to start
        calendar.setTime(sModel.getStartOfScope());
        oldDate = calendar.getTime();
        calendar.add(Calendar.DATE,1);
        i = 0; l = 0;
        while (enum.hasMoreElements()) {
          entry = (CalendarEntry)enum.nextElement();
          if (entry.getStart().before(oldDate)) {
            Assert.out.println("MonthlyPanel: illegal date in list");
            continue;
          } // end if
          while (!calendar.getTime().after(entry.getStart())) {
            oldDate = calendar.getTime();
            calendar.add(Calendar.DATE,1);
            i++; l = 0;
          } // end while
          if (i >= 31) {
            Assert.out.println("MonthlyPanel: date after end of spreadsheet");
            break;
          } // end if
          spreadSheet.setCell(1,i,l,entry);
          l++;
          while ((entry.getEnd().after(calendar.getTime())) && (i < 30)) {
            oldDate = calendar.getTime();
            calendar.add(Calendar.DATE,1);
            i++; l = 1;
            spreadSheet.setUplinkCell(1,i);
          } // end while
        } // end while
        spreadSheet.repaint();
      } // end if
    } // end if
  } // end update

  public void itemStateChanged(ItemEvent e) {
    Object aObj[] = spreadSheet.getSelectedObjects();
    Point point   = spreadSheet.getSelectedIndex();

    if (point == null) {
      sModel.setSelectedObject(aObj[0]);
    }
    else if ((point.x == 0) || (point.y >= nrOfDays)) {
      spreadSheet.unSelect();
      sModel.setSelectedObject(aObj[0]);
    }
    else {
      calendar.setTime(sModel.getStartOfScope());
      calendar.add(Calendar.DATE,point.y);
      Date start = new Date(calendar.getTime().getTime());
      calendar.add(Calendar.DATE,1);
      Date end = new Date(calendar.getTime().getTime());
      sModel.setSelectedObject(aObj[0],start,end);
    } // end if
  } // end itemStateChanged

} // end MonthlyPanel

/* ******************************************************************
   end of file
*********************************************************************/
