/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)            

   class      : SpreadSheetCanvas
   date       : 28.12.97
   description: a canvas that displays a very simple spreadsheet
   modified   : 03.02.98 - method unSelect added
                19.03.98 - bug in mouse selection removed

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class SpreadSheetCanvas extends DBCanvas
  implements ItemSelectable {

  // class variable declarations
  private final static int FONT_SIZE      = 10;
  private final static int DEF_COL_WIDTH  = 40;
  private final static int DEF_NR_OF_COLS = 2;
  private final static int DEF_NR_OF_ROWS = 10;

  private final static int DEF_ASCENT  = 10;
  private final static int DEF_LEADING = 0;
  private final static int DEF_DESCENT = 3;

  // instance variable declarations
  private Color fgColor;
  private Color bgColor;
  private boolean raiseCells;
  private int nrOfRows,nrOfCols;
  private int[] aColWidth;
  private int[] aColOfs;
  private int rowHeight         = 0;
  private int baseline          = 0;
  private int[] aRowOfs         = null;
  private Dimension size        = null;
  private boolean sizeAvailable = false;
  private SpreadSheetCanvas$Row[] aCols;
  private Vector listenerVector;
  private Object selectedObject = null;
  private int selX              = -1;
  private int selY              = -1;
  private int selLayer          = -1;
  private SpreadSheetCanvas me;        // a synonym for this for use in inner
                                       // class

  // inner classes
  class MouseEventHandler extends MouseAdapter {

    // methods
    public void mouseClicked(MouseEvent e) {
      int x = nrOfCols-1;
      //int y = e.getY()*nrOfRows/getSize().height;
      int y = e.getY()/rowHeight;
      int i;

      if (y >= nrOfRows) return;
      while (aColOfs[x] > e.getX()) x--;
      // deselect the old selection
      if (selX >= 0) highlight(null,selX,selY,selLayer);
      // update selection
      selX = x;
      selY = y;
      if (aCols[x].aRow[y] instanceof SpreadSheetCanvas$Uplink) {
        SpreadSheetCanvas$Uplink uplink = (SpreadSheetCanvas$Uplink)aCols[x].aRow[y];
        selY           -= uplink.offset;
        selLayer        = uplink.nrOfLayers-1;
        selectedObject  = uplink.object;
      }
      else if (aCols[x].aRow[y] instanceof Vector) {
        Vector vector = (Vector)aCols[x].aRow[y];
        int nrOfLayers = vector.size();
        selLayer       = nrOfLayers-1;
        while (aColOfs[x]+aColWidth[x]*selLayer/nrOfLayers > e.getX()) selLayer--;
        if ((selLayer == 0) &&
            (vector.elementAt(0) instanceof SpreadSheetCanvas$Uplink)) {
          SpreadSheetCanvas$Uplink uplink = (SpreadSheetCanvas$Uplink)vector.elementAt(0);
          selY           -= uplink.offset;
          selLayer        = uplink.nrOfLayers-1;
          selectedObject  = uplink.object;
        }
        else {
          selectedObject = vector.elementAt(selLayer);
        } // end if
      }
      else {
        selectedObject = aCols[x].aRow[y];
        selLayer       = 0;
      } // end if
      // update visible representation
      highlight(null,selX,selY,selLayer);
      // notify listeners
      ItemEvent ie  = new ItemEvent(me,ItemEvent.ITEM_STATE_CHANGED,
                                    selectedObject,ItemEvent.SELECTED);
      for (i=0; i<listenerVector.size(); i++) {
        ((ItemListener)listenerVector.elementAt(i)).itemStateChanged(ie);
      } // end for
    } // end mouseClicked

  } // end MouseEventHandler

  // constructors
  public SpreadSheetCanvas() {
    int[] aWidth = new int[DEF_NR_OF_COLS];
    int i;

    for (i=0; i<DEF_NR_OF_COLS; i++) aWidth[i] = DEF_COL_WIDTH;
    init(DEF_NR_OF_ROWS,aWidth,SystemColor.control,false);
  } // end constructor SpreadSheetCanvas

  public SpreadSheetCanvas(int nrOfRows, int nrOfCols) {
    int[] aWidth = new int[nrOfCols];
    int i;

    for (i=0; i<nrOfCols; i++) aWidth[i] = DEF_COL_WIDTH;
    init(nrOfRows,aWidth,SystemColor.control,false);
  } // end constructor SpreadSheetCanvas

  public SpreadSheetCanvas(int nrOfRows, int[] aWidth) {
    init(nrOfRows,aWidth,SystemColor.control,false);
  } // end constructor SpreadSheetCanvas

  public SpreadSheetCanvas(int nrOfRows, int[] aWidth,
                           Color color, boolean raiseCells) {
    init(nrOfRows,aWidth,color,raiseCells);
  } // end constructor SpreadSheetCanvas

  // methods
  private void init(int nrOfRows, int[] aWidth,
                    Color color, boolean raiseCells) {
    int i,t;

    this.bgColor    = color;
    this.raiseCells = raiseCells;
    this.fgColor    = SystemColor.controlText;
    this.nrOfRows   = nrOfRows;
    nrOfCols        = aWidth.length;
    this.aColWidth  = aWidth;
    aColOfs         = new int[nrOfCols];
    aCols           = new SpreadSheetCanvas$Row[nrOfCols];
    t               = 0;
    for (i=0; i<nrOfCols; i++) {
      aColOfs[i] = t; t += this.aColWidth[i];
      aCols[i]   = new SpreadSheetCanvas$Row(this.nrOfRows);
    } // end for
    listenerVector = new Vector();
    me             = this;
    addMouseListener(new MouseEventHandler());
    setBackground(this.bgColor);
  } // end init

  public void bufferedPaint(Graphics g) {
    Font font = new Font(g.getFont().getName(),
                         g.getFont().getStyle(),
                         FONT_SIZE);
    int i,j,k;
    int x,y,w;
    Object object;
    boolean isNull;
    boolean isVector;
    boolean isUplink;
    Vector vector;
    int nrOfLayers;
    Shape shape;

    if (getBufferEnabled()) {
      g.setColor(bgColor);
      g.fillRect(0,0,getSize().width,getSize().height);
    } // end if
    g.setFont(font);
    for (i=0; i<nrOfCols; i++) {
      for (j=0; j<nrOfRows; j++) {
        object     = aCols[i].aRow[j];
        isNull     = (object == null);
        isVector   = isNull ? false : (object instanceof Vector);
        if (isVector) {
          vector     = (Vector)object;
          nrOfLayers = vector.size();
        }
        else {
          vector     = null;
          nrOfLayers = 1;
        } // end if
        isUplink = (!isNull && !isVector &&
                    (object instanceof SpreadSheetCanvas$Uplink));
        g.setColor(bgColor);
        for (k=0; k<nrOfLayers; k++) {
        //  if (((selX == i) && (selY == j) && (selLayer == k)) ||
        //      ((isUplink) && (selY - ((SpreadSheetCanvas$Uplink)object).offset == j))) {
        //    g.fill3DRect(aColOfs[i]+aColWidth[i]*k/nrOfLayers,aRowOfs[j],
        //                 aColWidth[i]/nrOfLayers-1,rowHeight-1,
        //                 !isNull && raiseCells);
        //  }
        //  else {
            g.draw3DRect(aColOfs[i]+aColWidth[i]*k/nrOfLayers,aRowOfs[j],
                         aColWidth[i]/nrOfLayers-1,rowHeight-1,
                         !isNull && raiseCells);
        //  } // end if
        } // end for
        if (!isNull) {
          shape = g.getClip(); 
          g.setColor(fgColor);
          if (isVector) {
            for (k=0; k<nrOfLayers; k++) {
              if ((k == 0) &&
                  (vector.elementAt(k) instanceof SpreadSheetCanvas$Uplink)) {
                SpreadSheetCanvas$Uplink uplink = (SpreadSheetCanvas$Uplink)vector.elementAt(k);
                if ((uplink.nrOfLayers == 1) || (uplink.offset > 1)) {
                  g.setColor(bgColor);
                  g.setClip(shape);
                  g.drawRect(aColOfs[i]+aColWidth[i]*k/nrOfLayers+1,aRowOfs[j]-1,
                             aColWidth[i]/nrOfLayers-3,1);
                } // end if
              }
              else {
                g.setColor(fgColor);
                g.setClip(aColOfs[i]+aColWidth[i]*k/nrOfLayers+1,aRowOfs[j]+1,
                          aColWidth[i]/nrOfLayers-3,rowHeight-3);
                g.drawString(vector.elementAt(k).toString(),
                             aColOfs[i]+aColWidth[i]*k/nrOfLayers+1,
                             aRowOfs[j]+baseline+1);
              } // end if
            } // end for
          }
          else {
            x = aColOfs[i];
            y = aRowOfs[j];
            w = aColWidth[i];
            if (isUplink) {
              SpreadSheetCanvas$Uplink uplink = (SpreadSheetCanvas$Uplink)object;
              int upLayers = uplink.offset == 1 ? uplink.nrOfLayers : 1;
              g.setColor(bgColor);
              if (upLayers > 1) {
                g.fillRect(x+w*(upLayers-1)/upLayers+1,aRowOfs[j-1]+1,
                           w/upLayers-2,rowHeight);
                g.setColor(fgColor);
                g.drawString(uplink.object.toString(),x+1,y+baseline+1);
              }
              else {
                g.drawRect(x+1,y-1,w-3,1);
              } // end if
            }
            else {
              g.setClip(x+1,y+1,w-3,rowHeight-3);
              g.drawString(object.toString(),x+1,y+baseline+1);
            } // end if
          } // end if
          g.setClip(shape);
        } // end if
      } // end for
    } // end for

    if (selX >= 0) highlight(g,selX,selY,selLayer);
  } // end bufferedPaint

  private void highlight(Graphics g, int x, int y, int layer) {
    int nrOfLayers,curY;
    int i = 0;

    if (g == null) g = getGraphics();
    while ((i < 2) && (g != null)) {
      curY = y;
      g.setColor(fgColor);
      g.setXORMode(Color.white);
      if (aCols[x].aRow[curY] instanceof Vector) {
        nrOfLayers = ((Vector)aCols[x].aRow[curY]).size();
        g.fillRect(aColOfs[x]+aColWidth[x]*layer/nrOfLayers,aRowOfs[curY],
                   aColWidth[x]/nrOfLayers,rowHeight);
      }
      else {
        nrOfLayers = 1;
        g.fillRect(aColOfs[x],aRowOfs[curY],aColWidth[x],rowHeight);
      } // end if
      if (nrOfLayers == (layer+1)) {
        curY++;
       
        while ((curY < nrOfRows) &&
               (aCols[x].aRow[curY] instanceof SpreadSheetCanvas$Uplink)) {
          g.fillRect(aColOfs[x],aRowOfs[curY],aColWidth[x],rowHeight);
          curY++;
        } // end while
        if ((curY < nrOfRows) &&
            (aCols[x].aRow[curY] instanceof Vector) &&
            (((Vector)aCols[x].aRow[curY]).elementAt(0) instanceof SpreadSheetCanvas$Uplink)) {
          nrOfLayers = ((Vector)aCols[x].aRow[curY]).size();
          g.fillRect(aColOfs[x],aRowOfs[curY],
                     aColWidth[x]/nrOfLayers,rowHeight);
        } // end if
      } // end if
      g.setPaintMode();
      i++;
      g = getBufferGraphics();
    } // end while
  } // end highlight

  private Dimension calculateSize() {
    Graphics g = getGraphics();
    Font font;
    FontMetrics fm;
    int i,t;
    int ascent,descent,leading;

    if (g != null) {
      font = new Font(g.getFont().getName(),g.getFont().getStyle(),FONT_SIZE);
      g.setFont(font);
      fm            = g.getFontMetrics();
      ascent        = fm.getAscent();
      descent       = fm.getDescent();
      leading       = fm.getLeading();
      sizeAvailable = true;
    }
    else {
      ascent  = DEF_ASCENT;
      descent = DEF_DESCENT;
      leading = DEF_LEADING;
    } // end if
    rowHeight = ascent+descent+leading+2;
    t         = 0;
    aRowOfs   = new int[nrOfRows];
    for (i=0; i<nrOfRows; i++) {
      aRowOfs[i] = t; t += rowHeight;
    } // end for
    baseline = leading/2+ascent+1;
    return new Dimension(aColOfs[nrOfCols-1]+aColWidth[nrOfCols-1],
                         rowHeight*nrOfRows);
  } // end calculateSize

  public Object getCell(int x, int y, int layer) {
    if (aCols[x].aRow[y] instanceof Vector) {
      Vector vector = (Vector)aCols[x].aRow[y];
      if (layer < vector.size()) return vector.elementAt(layer);
        else return null;
    }
    else {
      return aCols[x].aRow[y];
    } // end if
  } // end getCell

  public void setCell(int x, int y, int layer, Object content) {
    if (aCols[x].aRow[y] == null) {
      aCols[x].aRow[y] = content;
    }
    else if (aCols[x].aRow[y] instanceof Vector) {
      Vector vector = (Vector)aCols[x].aRow[y];
      if (layer >= vector.size()) vector.addElement(content);
        else vector.setElementAt(content,layer);
    }
    else { // if no vector and not null
      if (layer == 0) {
        aCols[x].aRow[y] = content;
      }
      else {
        Vector v = new Vector();
        v.addElement(aCols[x].aRow[y]);
        v.addElement(content);
        aCols[x].aRow[y] = v;
      } // end if
    } // end if
    setBufferInvalid();
  } // end setCell

  public void deleteCell(int x, int y) {
    // full delete for all layers of a cell
    aCols[x].aRow[y] = null;
    setBufferInvalid();
  } // end deleteCell

  public void unSelect() {
    int i;

    if (selX >= 0) {
      highlight(null,selX,selY,selLayer);
      selX           = -1;
      selY           = -1;
      selLayer       = -1;
      selectedObject = null;
      // notify listeners
      ItemEvent ie  = new ItemEvent(this,ItemEvent.ITEM_STATE_CHANGED,
                                    selectedObject,ItemEvent.SELECTED);
      for (i=0; i<listenerVector.size(); i++) {
        ((ItemListener)listenerVector.elementAt(i)).itemStateChanged(ie);
      } // end for
    } // end if
    setBufferInvalid();
  } // end unSelect

  public void setUplinkCell(int x, int y) {
    aCols[x].aRow[y] = new SpreadSheetCanvas$Uplink(aCols[x].aRow[y-1]);
    setBufferInvalid();
  } // end setUplinkCell

  public synchronized void addItemListener(ItemListener l) {
    listenerVector.addElement(l);
  } // end addItemListener

  public synchronized void removeItemListener(ItemListener l) {
    listenerVector.removeElement(l);
  } // end addItemListener

  public Object[] getSelectedObjects() {
    Object[] objects = new Object[1];
    objects[0]       = selectedObject;
    return objects;
  } // end getSelectedObject

  public Point getSelectedIndex() {
    if (selX < 0)
      return null;
    else
      return new Point(selX,selY);
    // end if
  } // end getSelectedIndex

  public int getSelectedLayer() {
    return selLayer;
  } // end getSelectedLayer

  public Dimension getPreferredSize() {
    if (!sizeAvailable) size = calculateSize();
    return size;
  } // end getPreferredSize

  public Dimension getMinimumSize() {
    return getPreferredSize();
  } // end getMinimumSize

  public Dimension getMaximumSize() {
    return getPreferredSize();
  } // end getMaximumSize

} // end SpreadSheetCanvas

/* ******************************************************************
   class declaration
*********************************************************************/

class SpreadSheetCanvas$Row {

  // instance variable declarations
  public Object[] aRow;
  // constructors
  public SpreadSheetCanvas$Row(int length) {
    int i;

    aRow = new Object[length];
    for (i=0; i<length; i++) aRow[i] = null;
  } // end constructor SpreadSheetCanvas$Row

} // end SpreadSheetCanvas$Row

class SpreadSheetCanvas$Uplink {

  // instance variable declarations
  public Object object;
  public int nrOfLayers;
  public int offset;

  // constructors
  public SpreadSheetCanvas$Uplink(Object object) {
    if (object instanceof Vector) {
      Vector vector = (Vector)object;
      nrOfLayers    = vector.size();
      this.object   = vector.elementAt(nrOfLayers-1);
      offset        = 1;
    }
    else if (object instanceof SpreadSheetCanvas$Uplink) {
      SpreadSheetCanvas$Uplink uplink = (SpreadSheetCanvas$Uplink)object;
      nrOfLayers  = uplink.nrOfLayers;
      this.object = uplink.object;
      offset      = uplink.offset+1;
    }
    else {
      nrOfLayers  = 1;
      this.object = object;
      offset        = 1;
    } // end if
  } // end constructor SpreadSheetCanvas$Uplink

} // end SpreadSheetCanvas$Uplink

/* ******************************************************************
   end of file
*********************************************************************/
